@@ -10,19 +10,19 @@ from math import cos, radians, sin, sqrt, acos
 from copy import deepcopy
 
 def create_face(block:Block,imin:int,imax:int,jmin:int,jmax:int,kmin:int,kmax:int) -> Face:
-    """Creates a face/surface
+    """Creates a face/surface from IJK bounds. Face = either i is constant or j constant or k constant. 
 
     Args:
-        block (Block): [description]
-        imin (int): [description]
-        imax (int): [description]
-        jmin (int): [description]
-        jmax (int): [description]
-        kmin (int): [description]
-        kmax (int): [description]
+        block (Block): Block object containing X,Y,Z as 3 dimensional arrays 
+        imin (int): Minimum I-index
+        imax (int): Maximum I-index
+        jmin (int): Minimum J-index
+        jmax (int): Maximum J-index
+        kmin (int): Minimum K-index
+        kmax (int): Maximum K-index
 
     Returns:
-        Face: [description]
+        Face: Face object with vertices of I,J,K
     """
     f = Face(4)
     if imin==imax:
@@ -131,63 +131,95 @@ def find_periodicity(blocks:List[Block],outer_faces:List, periodic_direction:str
             # https://www.geeksforgeeks.org/python-program-to-get-all-unique-combinations-of-two-lists/ 
             for b1_surf_indx in range(len(outer_faces[b1]['surfaces'])):
                 for b2_surf_indx in range(len(outer_faces[b2]['surfaces'])):
+                    # Creates two surfaces from b1_surf and b2_surf
                     b1_surf = outer_faces[b1]['surfaces'][b1_surf_indx]
                     b2_surf = outer_faces[b2]['surfaces'][b2_surf_indx]
 
-                    if (b1_surf['KMIN'] == b1_surf['KMAX']) and (b2_surf['KMIN'] == b2_surf['KMAX']):
-                        surface1 = create_face(blocks[b1], b1_surf['IMIN'], b1_surf['IMAX'], 
+                    surface1 = create_face(blocks[b1], b1_surf['IMIN'], b1_surf['IMAX'], 
                                                             b1_surf['JMIN'], b1_surf['JMAX'], 
                                                             b1_surf['KMIN'], b1_surf['KMAX'])
 
-                        surface2 = create_face(blocks[b2], b2_surf['IMIN'], b2_surf['IMAX'], 
+                    surface2 = create_face(blocks[b2], b2_surf['IMIN'], b2_surf['IMAX'], 
                                                             b2_surf['JMIN'], b2_surf['JMAX'], 
                                                             b2_surf['KMIN'], b2_surf['KMAX'])
-
-                        # Step 1: Try rotating surface 1 and check for intersections 
-                        block1_rotated = rotate_block(blocks[b1], rotation_matrix)
-                        # write_plot3D('test_rotated.xyz',[block1_rotated],binary=True) # debug purposes
-                        df, split_face1, split_face2 = get_face_intersection(surface1,surface2,block1_rotated,blocks[b2],tol=1E-5)
-                        if len(df)>0:                          # Try with first block rotated
-                            if periodic_direction == "i":
-                                df['j1'] += surface1.JMIN
-                                df['k1'] += surface1.KMIN
-                                df['j2'] += surface2.JMIN
-                                df['k2'] += surface2.KMIN
-
-                            if periodic_direction == "j":
-                                df['i1'] += surface1.IMIN
-                                df['k1'] += surface1.KMIN
-                                df['i2'] += surface2.IMIN
-                                df['k2'] += surface2.KMIN
-
-                            if periodic_direction == "k":
-                                df['i1'] += surface1.IMIN
-                                df['j1'] += surface1.JMIN
-                                df['i2'] += surface2.IMIN
-                                df['j2'] += surface2.JMIN
-                        elif len(df)==0:                        # Try with second block rotated
+                    
+                    
+                    if periodic_direction == "i" and (b1_surf['IMIN'] == b1_surf['IMAX']) and (b2_surf['IMIN'] == b2_surf['IMAX']):
+                        block1_rotated = rotate_block(blocks[b1], rotation_matrix) 
+                        df, _, _ = get_face_intersection(surface1,surface2,block1_rotated,blocks[b2],tol=1E-5)
+                            
+                        if len(df)==0:                        # Try with second block rotated
                             # Step 2: if Step 1 does not yield a match then rotate surface 2 and check for intersections 
                             block2_rotated = rotate_block(blocks[b2], rotation_matrix)
-                            df, split_face1, split_face2 = get_face_intersection(surface1,surface2,blocks[b1],block2_rotated,tol=1E-5)
-                            if periodic_direction == "i":
-                                df['j1'] += surface1.JMIN
-                                df['k1'] += surface1.KMIN
-                                df['j2'] += surface2.JMIN
-                                df['k2'] += surface2.KMIN
-
-                            if periodic_direction == "j":
-                                df['i1'] += surface1.IMIN
-                                df['k1'] += surface1.KMIN
-                                df['i2'] += surface2.IMIN
-                                df['k2'] += surface2.KMIN
-
-                            if periodic_direction == "k":
-                                df['i1'] += surface1.IMIN
-                                df['j1'] += surface1.JMIN
-                                df['i2'] += surface2.IMIN
-                                df['j2'] += surface2.JMIN
-                            
+                            df, _, _ = get_face_intersection(surface1,surface2,blocks[b1],block2_rotated,tol=1E-5)
+                           
                         if len(df)>0:
+                            df['j1'] += surface1.JMIN           
+                            df['k1'] += surface1.KMIN
+                            df['j2'] += surface2.JMIN
+                            df['k2'] += surface2.KMIN
+                            periodic_surfaces.append({
+                                                        'block1':{
+                                                                'index':b1,
+                                                                'IMIN':df.iloc[0]['i1'],'JMIN':df.iloc[0]['j1'],'KMIN':df.iloc[0]['k1'],
+                                                                'IMAX':df.iloc[-1]['i1'],'JMAX':df.iloc[-1]['j1'],'KMAX':df.iloc[-1]['k1']
+                                                                },
+                                                        'block2':{
+                                                                'index':b2,
+                                                                'IMIN':df.iloc[0]['i2'],'JMIN':df.iloc[0]['j2'],'KMIN':df.iloc[0]['k2'],
+                                                                'IMAX':df.iloc[-1]['i2'],'JMAX':df.iloc[-1]['j2'],'KMAX':df.iloc[-1]['k2']
+                                                                },
+                                                        })
+                            # query faces to remove
+                            block_outer_faces_to_remove.append({'block_indx':b1,'surface_indx':b1_surf_indx})
+                            block_outer_faces_to_remove.append({'block_indx':b2,'surface_indx':b2_surf_indx})
+
+                    elif periodic_direction == "j" and (b1_surf['JMIN'] == b1_surf['JMAX']) and (b2_surf['JMIN'] == b2_surf['JMAX']):
+                        block1_rotated = rotate_block(blocks[b1], rotation_matrix) # Rotate the block 
+                        df, _, _ = get_face_intersection(surface1,surface2,block1_rotated,blocks[b2],tol=1E-5)
+
+                        if len(df)==0:                        # Try with second block rotated
+                            # Step 2: if Step 1 does not yield a match then rotate surface 2 and check for intersections 
+                            block2_rotated = rotate_block(blocks[b2], rotation_matrix)
+                            df, _, _ = get_face_intersection(surface1,surface2,blocks[b1],block2_rotated,tol=1E-5)
+
+                        if len(df)>0:
+                            df['i1'] += surface1.IMIN
+                            df['k1'] += surface1.KMIN
+                            df['i2'] += surface2.IMIN
+                            df['k2'] += surface2.KMIN
+                            periodic_surfaces.append({
+                                                        'block1':{
+                                                            'index':b1,
+                                                            'IMIN':df.iloc[0]['i1'],'JMIN':df.iloc[0]['j1'],'KMIN':df.iloc[0]['k1'],
+                                                            'IMAX':df.iloc[-1]['i1'],'JMAX':df.iloc[-1]['j1'],'KMAX':df.iloc[-1]['k1']
+                                                            },
+                                                        'block2':{
+                                                                'index':b2,
+                                                                'IMIN':df.iloc[0]['i2'],'JMIN':df.iloc[0]['j2'],'KMIN':df.iloc[0]['k2'],
+                                                                'IMAX':df.iloc[-1]['i2'],'JMAX':df.iloc[-1]['j2'],'KMAX':df.iloc[-1]['k2']
+                                                            },
+                                                        })
+                            # query faces to remove
+                            block_outer_faces_to_remove.append({'block_indx':b1,'surface_indx':b1_surf_indx})
+                            block_outer_faces_to_remove.append({'block_indx':b2,'surface_indx':b2_surf_indx})
+
+                    # If the k's are constant then rotate the block by a rotation matrix
+                    elif periodic_direction == "k" and (b1_surf['KMIN'] == b1_surf['KMAX']) and (b2_surf['KMIN'] == b2_surf['KMAX']):
+                        block1_rotated = rotate_block(blocks[b1], rotation_matrix) # Rotate block 1
+                        
+                        # write_plot3D('test_rotated.xyz',[block1_rotated],binary=True) # debug purposes
+                        # Step 1: Try rotating surface 1 and check for intersections 
+                        df, _, _ = get_face_intersection(surface1,surface2,block1_rotated,blocks[b2],tol=1E-5)
+                        if len(df)==0:
+                            block2_rotated = rotate_block(blocks[b2], rotation_matrix) # Rotate block 2
+                            df, _, _ = get_face_intersection(surface1,surface2,blocks[b1],block2_rotated,tol=1E-5)
+                        
+                        if len(df)>0:
+                            df['i1'] += surface1.IMIN
+                            df['j1'] += surface1.JMIN
+                            df['i2'] += surface2.IMIN
+                            df['j2'] += surface2.JMIN
                             # Here we save the periodicity 
                             periodic_surfaces.append({
                                                         'block1':{
@@ -202,7 +234,7 @@ def find_periodicity(blocks:List[Block],outer_faces:List, periodic_direction:str
                             # query faces to remove
                             block_outer_faces_to_remove.append({'block_indx':b1,'surface_indx':b1_surf_indx})
                             block_outer_faces_to_remove.append({'block_indx':b2,'surface_indx':b2_surf_indx})
-        
+            
         # Lets remove the outer faces
         for b in range(len(outer_faces)):
             outer_faces_to_keep.append({'index':b, 'surfaces':[]})
